string GlobalVersion = "1.0";

/*

   Indicator plots Indexed Rate of Change values 

*/


//---- indicator settings
#property  indicator_separate_window
#property  indicator_minimum 0.0
#property  indicator_maximum 100.0
#property  indicator_buffers 8
#property  indicator_color1  Blue
#property  indicator_color2  Red
#property  indicator_color3  Black
#property  indicator_color4  Green
#property  indicator_width1  1
//---- indicator parameters
extern int Period1=21;
extern int Period2=21;
extern int PeriodCross=8;
extern int PeriodFirst=30; 
//---- indicator buffers
double     FirstBuffer[];
double     SignalBuffer[];
double     SmoothedRateOfChangeBuffer[];
double     PosBuffer[];
double     NegBuffer[];
//   double     AveragePosBuffer[];
//   double     AverageNegBuffer[];
double     TempBufferOne[];
double     TempBufferTwo[];
double     SmoothedROCBuffer[];

bool       FirstTime = true;
int        SubWindow = 0;
string     SubWindowName;

int deinit()
{
   if(SubWindow > 0)
         ObjectsDeleteAll(SubWindow);
}


//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
  int i;
  
//---- name for DataWindow and indicator subwindow label
   SubWindowName = StringConcatenate("SmoothedROC ", GlobalVersion);
   IndicatorShortName(SubWindowName);
    
    
//---- drawing settings
//   SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexStyle(0,DRAW_LINE);
   SetIndexStyle(1,DRAW_LINE,STYLE_DOT);
   SetIndexStyle(2,DRAW_NONE);
   SetIndexStyle(3,DRAW_NONE);
   SetIndexStyle(4,DRAW_NONE);
   SetIndexStyle(5,DRAW_NONE);
   SetIndexStyle(6,DRAW_NONE);
   SetIndexStyle(7,DRAW_NONE);
//   SetIndexDrawBegin(1,PeriodCross);
   IndicatorDigits(Digits+4);
//---- indicator buffers mapping
   SetIndexBuffer(0,SmoothedROCBuffer);
   SetIndexBuffer(1,SignalBuffer);
   SetIndexBuffer(2,FirstBuffer);
   SetIndexBuffer(3,SmoothedRateOfChangeBuffer);
   SetIndexBuffer(4,PosBuffer);
   SetIndexBuffer(5,NegBuffer);
   SetIndexBuffer(6,TempBufferOne);
   SetIndexBuffer(7,TempBufferTwo);   
   SetIndexLabel(0,"SmoothedROC");
   SetIndexLabel(1,"Signal");
   SetIndexLabel(2,"First");
   SetIndexLabel(3,"Smoothed");
   SetIndexLabel(4,"Positives");
   SetIndexLabel(5,"Negatives");
   SetIndexLabel(6,"TempBuffer");
   SetIndexLabel(7,"TempBufferTwo");

   
//---- initialization done
   return(0);
  }

int start()
  {
   int i, limit;
   int counted_bars=IndicatorCounted();
   double OldEma = 0;
   string  Name85, Name15;
   
   if(FirstTime == true)
   {
      FirstTime = false; 
      SubWindow = WindowFind(SubWindowName);
      Print(SubWindowName, " started in sub window number ", SubWindow);     
                
      Print("Starting SmoothedROC, Counted=", counted_bars, " of total candles=", Bars);
      Print("Periods = ", Period1, " ", Period2, " ", PeriodCross);
              
      Name15 = "Level15-" + SubWindow;   
      Name85 = "Level85-" + SubWindow;
      
      ObjectCreate(Name85,OBJ_TREND, SubWindow, Time[Bars], 85.0, Time[0], 85.0);
      ObjectSet   (Name85,OBJPROP_COLOR,LightSlateGray);
      ObjectCreate(Name15,OBJ_TREND, SubWindow, Time[Bars], 15.0, Time[0], 15.0);
      ObjectSet   (Name15,OBJPROP_COLOR,LightSlateGray);
      
      Print("Create lines in window ", SubWindow, " last error=", GetLastError() );      
     
   }
   
   
//---- last counted bar will be recounted
   if(counted_bars>0) counted_bars--;
   limit=Bars-counted_bars;


   for(i=0; i<limit; i++)
      TempBufferOne[i]= iMA(NULL,0,PeriodFirst,0,MODE_EMA,PRICE_CLOSE,i);


   for(i=0; i<limit; i++)
                                             //  use ema from Period1 ago if it is in range, else use current one
                                             //  only impacts very first PeriodFirst candles in whole chart 
      {                                      
         if(i+PeriodFirst < Bars)
               FirstBuffer[i]=(  Close[i] -  TempBufferOne[i] )
                                          / TempBufferOne[i+PeriodFirst] ; 
         else                                                
               FirstBuffer[i]=(  Close[i] -  TempBufferOne[i] )
                                          / TempBufferOne[i] ; 
      }
                              
                              
   for(i=0; i<limit; i++)
       SmoothedRateOfChangeBuffer [i] = iMAOnArray(FirstBuffer, Bars, Period1,0,MODE_EMA,i);
                              

       
       
//  separate up and down changes within smoothed roc buffer

                              
   for(i=0; i<limit; i++)
       if (SmoothedRateOfChangeBuffer[i+1] < SmoothedRateOfChangeBuffer[i])
          {
            PosBuffer[i] = SmoothedRateOfChangeBuffer[i] - SmoothedRateOfChangeBuffer[i+1];
            NegBuffer[i] = 0.0;
          }
       else 
          {
            PosBuffer[i] = 0.0;
            NegBuffer[i] = SmoothedRateOfChangeBuffer[i] - SmoothedRateOfChangeBuffer[i+1];
          }
                              
   for(i=0; i<limit; i++)
      {
         if (   iMAOnArray(PosBuffer, Bars, Period2,0,MODE_EMA,i)
              - iMAOnArray(NegBuffer, Bars, Period2,0,MODE_EMA,i) == 0.0 )
              {
               Print("avoiding divide by zero for candle ", i,
                      " Pos[i]=", iMAOnArray(PosBuffer, Bars, Period2,0,MODE_EMA,i),
                      " Neg[i]=", iMAOnArray(NegBuffer, Bars, Period2,0,MODE_EMA,i) );
               SmoothedROCBuffer [i] = 50;
             }         
         else 
            {
          SmoothedROCBuffer [i] = 100.0 * iMAOnArray(PosBuffer, Bars, Period2,0,MODE_EMA,i) /
                               (    iMAOnArray(PosBuffer, Bars, Period2,0,MODE_EMA,i)
                                  - iMAOnArray(NegBuffer, Bars, Period2,0,MODE_EMA,i) );                                 
            }
      }
                              

              
   for(i=0; i<limit; i++)
       {
         SignalBuffer [i] = iMAOnArray(SmoothedROCBuffer, Bars, PeriodCross,0,MODE_EMA,i);       
       }       
      

//---- done
   return(0);
  }
//+------------------------------------------------------------------+